@@ -1,11 +1,13 @@
 #ifndef _BUFFER_H
 #define _BUFFER_H
 
-#define CHUNK_SIZE (64 * 1024)
+/* must be a multiple of any supported frame size */
+#define CHUNK_SIZE (60 * 1024)
 
 extern unsigned int buffer_nr_chunks;
 
 void buffer_init(void);
+void buffer_free(void);
 int buffer_get_rpos(char **pos);
 int buffer_get_wpos(char **pos);
 void buffer_consume(int count);
