@@ -26,6 +26,7 @@
 #include "utils.h"
 #include "cmus.h"
 #include "list.h"
+#include "mergesort.h"
 #include "misc.h"
 #include "debug.h"
 #include "ui_curses.h"
@@ -36,11 +37,14 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <sys/select.h>
+#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
 #include <dlfcn.h>
+#include <strings.h>
 
 struct input_plugin {
 	const struct input_plugin_ops *ops;
@@ -52,6 +56,10 @@ struct input_plugin {
 
 	/* cached duration, -1 = unset */
 	int duration;
+	/* cached bitrate, -1 = unset */
+	long bitrate;
+	/* cached codec, NULL = unset */
+	char *codec;
 
 	/*
 	 * pcm is converted to 16-bit signed little-endian stereo
@@ -72,6 +80,7 @@ struct ip {
 	char *name;
 	void *handle;
 
+	int priority;
 	const char * const *extensions;
 	const char * const *mime_types;
 	const struct input_plugin_ops *ops;
@@ -105,21 +114,20 @@ static const char *get_extension(const char *filename)
 	return NULL;
 }
 
-static const struct input_plugin_ops *get_ops_by_filename(const char *filename)
+static const struct input_plugin_ops *get_ops_by_extension(const char *ext, struct list_head **headp)
 {
-	struct ip *ip;
-	const char *ext;
+	struct list_head *node = *headp;
 
-	ext = get_extension(filename);
-	if (ext == NULL)
-		return NULL;
-	list_for_each_entry(ip, &ip_head, node) {
+	for (node = node->next; node != &ip_head; node = node->next) {
+		struct ip *ip = list_entry(node, struct ip, node);
 		const char * const *exts = ip->extensions;
 		int i;
 
 		for (i = 0; exts[i]; i++) {
-			if (strcasecmp(ext, exts[i]) == 0)
+			if (strcasecmp(ext, exts[i]) == 0 || strcmp("any", exts[i]) == 0) {
+				*headp = node;
 				return ip->ops;
+			}
 		}
 	}
 	return NULL;
@@ -141,6 +149,25 @@ static const struct input_plugin_ops *get_ops_by_mime_type(const char *mime_type
 	return NULL;
 }
 
+static void keyvals_add_basic_auth(struct growing_keyvals *c,
+				   const char *user,
+				   const char *pass,
+				   const char *header)
+{
+	char buf[256];
+	char *encoded;
+
+	snprintf(buf, sizeof(buf), "%s:%s", user, pass);
+	encoded = base64_encode(buf);
+	if (encoded == NULL) {
+		d_print("couldn't base64 encode '%s'\n", buf);
+	} else {
+		snprintf(buf, sizeof(buf), "Basic %s", encoded);
+		free(encoded);
+		keyvals_add(c, header, xstrdup(buf));
+	}
+}
+
 static int do_http_get(struct http_get *hg, const char *uri, int redirections)
 {
 	GROWING_KEYVALS(h);
@@ -161,22 +188,12 @@ static int do_http_get(struct http_get *hg, const char *uri, int redirections)
 		return -IP_ERROR_ERRNO;
 
 	keyvals_add(&h, "Host", xstrdup(hg->uri.host));
+	if (hg->proxy && hg->proxy->user && hg->proxy->pass)
+		keyvals_add_basic_auth(&h, hg->proxy->user, hg->proxy->pass, "Proxy-Authorization");
 	keyvals_add(&h, "User-Agent", xstrdup("cmus/" VERSION));
 	keyvals_add(&h, "Icy-MetaData", xstrdup("1"));
-	if (hg->uri.user && hg->uri.pass) {
-		char buf[256];
-		char *encoded;
-
-		snprintf(buf, sizeof(buf), "%s:%s", hg->uri.user, hg->uri.pass);
-		encoded = base64_encode(buf);
-		if (encoded == NULL) {
-			d_print("couldn't base64 encode '%s'\n", buf);
-		} else {
-			snprintf(buf, sizeof(buf), "Basic %s", encoded);
-			free(encoded);
-			keyvals_add(&h, "Authorization", xstrdup(buf));
-		}
-	}
+	if (hg->uri.user && hg->uri.pass)
+		keyvals_add_basic_auth(&h, hg->uri.user, hg->uri.pass, "Authorization");
 	keyvals_terminate(&h);
 
 	rc = http_get(hg, h.keyvals, http_read_timeout);
@@ -248,7 +265,7 @@ static int setup_remote(struct input_plugin *ip, const struct keyval *headers, i
 	}
 
 	ip->data.fd = sock;
-	ip->data.metadata = xmalloc(16 * 255 + 1);
+	ip->data.metadata = xnew(char, 16 * 255 + 1);
 
 	val = keyvals_get_val(headers, "icy-metaint");
 	if (val) {
@@ -344,7 +361,7 @@ static int open_remote(struct input_plugin *ip)
 	if (val) {
 		int i;
 
-		for (i = 0; i < sizeof(pl_mime_types) / sizeof(pl_mime_types[0]); i++) {
+		for (i = 0; i < N_ELEMENTS(pl_mime_types); i++) {
 			if (!strcasecmp(val, pl_mime_types[i])) {
 				d_print("Content-Type: %s\n", val);
 				http_get_free(&hg);
@@ -358,17 +375,75 @@ static int open_remote(struct input_plugin *ip)
 	return rc;
 }
 
+static void ip_init(struct input_plugin *ip, char *filename)
+{
+	memset(ip, 0, sizeof(*ip));
+	ip->http_code = -1;
+	ip->pcm_convert_scale = -1;
+	ip->duration = -1;
+	ip->bitrate = -1;
+	ip->codec = NULL;
+	ip->data.fd = -1;
+	ip->data.filename = filename;
+	ip->data.remote = is_url(filename);
+}
+
+static void ip_reset(struct input_plugin *ip, int close_fd)
+{
+	int fd = ip->data.fd;
+	free(ip->data.metadata);
+	ip_init(ip, ip->data.filename);
+	if (fd != -1) {
+		if (close_fd)
+			close(fd);
+		else {
+			lseek(fd, 0, SEEK_SET);
+			ip->data.fd = fd;
+		}
+	}
+}
+
 static int open_file(struct input_plugin *ip)
 {
-	ip->ops = get_ops_by_filename(ip->data.filename);
-	if (ip->ops == NULL)
+	const struct input_plugin_ops *ops;
+	struct list_head *head = &ip_head;
+	const char *ext;
+	int rc = 0;
+
+	ext = get_extension(ip->data.filename);
+	if (!ext)
 		return -IP_ERROR_UNRECOGNIZED_FILE_TYPE;
+
+	ops = get_ops_by_extension(ext, &head);
+	if (!ops)
+		return -IP_ERROR_UNRECOGNIZED_FILE_TYPE;
+
 	ip->data.fd = open(ip->data.filename, O_RDONLY);
-	if (ip->data.fd == -1) {
-		ip->ops = NULL;
+	if (ip->data.fd == -1)
 		return -IP_ERROR_ERRNO;
+
+	while (1) {
+		ip->ops = ops;
+		rc = ip->ops->open(&ip->data);
+		if (rc != -IP_ERROR_UNSUPPORTED_FILE_TYPE)
+			break;
+
+		ops = get_ops_by_extension(ext, &head);
+		if (!ops)
+			break;
+
+		ip_reset(ip, 0);
+		d_print("fallback: try next plugin for `%s'\n", ip->data.filename);
 	}
-	return 0;
+
+	return rc;
+}
+
+static int sort_ip(const struct list_head *a_, const struct list_head *b_)
+{
+	const struct ip *a = list_entry(a_, struct ip, node);
+	const struct ip *b = list_entry(b_, struct ip, node);
+	return b->priority - a->priority;
 }
 
 void ip_load_plugins(void)
@@ -381,11 +456,12 @@ void ip_load_plugins(void)
 		error_msg("couldn't open directory `%s': %s", plugin_dir, strerror(errno));
 		return;
 	}
-	while ((d = readdir(dir)) != NULL) {
+	while ((d = (struct dirent *) readdir(dir)) != NULL) {
 		char filename[256];
 		struct ip *ip;
 		void *so;
 		char *ext;
+		const int *priority_ptr;
 
 		if (d->d_name[0] == '.')
 			continue;
@@ -399,41 +475,33 @@ void ip_load_plugins(void)
 
 		so = dlopen(filename, RTLD_NOW);
 		if (so == NULL) {
-			error_msg("%s", dlerror());
+			d_print("%s: %s\n", filename, dlerror());
 			continue;
 		}
 
 		ip = xnew(struct ip, 1);
 
+		priority_ptr = dlsym(so, "ip_priority");
 		ip->extensions = dlsym(so, "ip_extensions");
 		ip->mime_types = dlsym(so, "ip_mime_types");
 		ip->ops = dlsym(so, "ip_ops");
-		if (!ip->extensions || !ip->mime_types || !ip->ops) {
+		if (!priority_ptr || !ip->extensions || !ip->mime_types || !ip->ops) {
 			error_msg("%s: missing symbol", filename);
 			free(ip);
 			dlclose(so);
 			continue;
 		}
+		ip->priority = *priority_ptr;
 
 		ip->name = xstrndup(d->d_name, ext - d->d_name);
 		ip->handle = so;
 
 		list_add_tail(&ip->node, &ip_head);
 	}
+	list_mergesort(&ip_head, sort_ip);
 	closedir(dir);
 }
 
-static void ip_init(struct input_plugin *ip, char *filename)
-{
-	memset(ip, 0, sizeof(*ip));
-	ip->http_code = -1;
-	ip->pcm_convert_scale = -1;
-	ip->duration = -1;
-	ip->data.fd = -1;
-	ip->data.filename = filename;
-	ip->data.remote = is_url(filename);
-}
-
 struct input_plugin *ip_new(const char *filename)
 {
 	struct input_plugin *ip = xnew(struct input_plugin, 1);
@@ -456,9 +524,11 @@ int ip_open(struct input_plugin *ip)
 
 	BUG_ON(ip->open);
 
-	/* set fd and ops */
+	/* set fd and ops, call ops->open */
 	if (ip->data.remote) {
 		rc = open_remote(ip);
+		if (rc == 0)
+			rc = ip->ops->open(&ip->data);
 	} else {
 		rc = open_file(ip);
 	}
@@ -466,17 +536,7 @@ int ip_open(struct input_plugin *ip)
 	if (rc) {
 		d_print("opening `%s' failed: %d %s\n", ip->data.filename, rc,
 				rc == -1 ? strerror(errno) : "");
-		return rc;
-	}
-
-	rc = ip->ops->open(&ip->data);
-	if (rc) {
-		d_print("opening file `%s' failed: %d %s\n", ip->data.filename, rc,
-				rc == -1 ? strerror(errno) : "");
-		if (ip->data.fd != -1)
-			close(ip->data.fd);
-		free(ip->data.metadata);
-		ip_init(ip, ip->data.filename);
+		ip_reset(ip, 1);
 		return rc;
 	}
 	ip->open = 1;
@@ -576,7 +636,7 @@ int ip_read(struct input_plugin *ip, char *buffer, int count)
 		return rc;
 	}
 
-	BUG_ON((rc & ~((unsigned int)sf_get_frame_size(ip->data.sf) - 1U)) != rc);
+	BUG_ON(rc % sf_get_frame_size(ip->data.sf) != 0);
 
 	sample_size = sf_get_sample_size(ip->data.sf);
 	if (ip->pcm_convert_in_place != NULL)
@@ -643,6 +703,26 @@ int ip_duration(struct input_plugin *ip)
 	return ip->duration;
 }
 
+int ip_bitrate(struct input_plugin *ip)
+{
+	if (ip->data.remote)
+		return -1;
+	if (ip->bitrate == -1)
+		ip->bitrate = ip->ops->bitrate(&ip->data);
+	if (ip->bitrate < 0)
+		return -1;
+	return ip->bitrate;
+}
+
+char *ip_codec(struct input_plugin *ip)
+{
+	if (ip->data.remote)
+		return NULL;
+	if (!ip->codec)
+		ip->codec = ip->ops->codec(&ip->data);
+	return ip->codec;
+}
+
 sample_format_t ip_get_sf(struct input_plugin *ip)
 {
 	BUG_ON(!ip->open);
@@ -692,6 +772,10 @@ char *ip_get_error_msg(struct input_plugin *ip, int rc, const char *arg)
 		snprintf(buffer, sizeof(buffer),
 				"%s: unrecognized filename extension", arg);
 		break;
+	case IP_ERROR_UNSUPPORTED_FILE_TYPE:
+		snprintf(buffer, sizeof(buffer),
+				"%s: unsupported file format", arg);
+		break;
 	case IP_ERROR_FUNCTION_NOT_SUPPORTED:
 		snprintf(buffer, sizeof(buffer),
 				"%s: function not supported", arg);
@@ -766,7 +850,7 @@ void ip_dump_plugins(void)
 
 	printf("Input Plugins: %s\n", plugin_dir);
 	list_for_each_entry(ip, &ip_head, node) {
-		printf("  %s:\n    File Types:", ip->name);
+		printf("  %s:\n    Priority: %d\n    File Types:", ip->name, ip->priority);
 		for (i = 0; ip->extensions[i]; i++)
 			printf(" %s", ip->extensions[i]);
 		printf("\n    MIME Types:");
