@@ -22,9 +22,13 @@
 #include "prog.h"
 #include "output.h"
 #include "config/datadir.h"
+#include "track_info.h"
+#include "cache.h"
+#include "debug.h"
 
 #include <stdio.h>
 #include <errno.h>
+#include <strings.h>
 
 #if defined(__sun__)
 #include <ncurses.h>
@@ -37,8 +41,9 @@
 char *output_plugin = NULL;
 char *status_display_program = NULL;
 char *server_password;
-int auto_reshuffle = 0;
+int auto_reshuffle = 1;
 int confirm_run = 1;
+int resume_cmus = 0;
 int show_hidden = 0;
 int show_remaining_time = 0;
 int set_term_title = 1;
@@ -184,30 +189,39 @@ static void set_id3_default_charset(unsigned int id, const char *buf)
 	id3_default_charset = xstrdup(buf);
 }
 
-static const char * const valid_sort_keys[] = {
-	"artist",
-	"album",
-	"title",
-	"tracknumber",
-	"discnumber",
-	"date",
-	"genre",
-	"comment",
-	"filename",
-	"albumartist",
-	"filemtime",
-	NULL
+static const struct {
+	const char *str;
+	sort_key_t key;
+} sort_key_map[] = {
+	{ "artist",		SORT_ARTIST		},
+	{ "album",		SORT_ALBUM		},
+	{ "title",		SORT_TITLE		},
+	{ "tracknumber",	SORT_TRACKNUMBER	},
+	{ "discnumber",		SORT_DISCNUMBER		},
+	{ "date",		SORT_DATE		},
+	{ "genre",		SORT_GENRE		},
+	{ "comment",		SORT_COMMENT		},
+	{ "albumartist",	SORT_ALBUMARTIST	},
+	{ "filename",		SORT_FILENAME		},
+	{ "filemtime",		SORT_FILEMTIME		},
+	{ "rg_track_gain",	SORT_RG_TRACK_GAIN	},
+	{ "rg_track_peak",	SORT_RG_TRACK_PEAK	},
+	{ "rg_album_gain",	SORT_RG_ALBUM_GAIN	},
+	{ "rg_album_peak",	SORT_RG_ALBUM_PEAK	},
+	{ "bitrate",		SORT_BITRATE		},
+	{ "codec",		SORT_CODEC		},
+	{ NULL,                 SORT_INVALID            }
 };
 
-static const char **parse_sort_keys(const char *value)
+static sort_key_t *parse_sort_keys(const char *value)
 {
-	const char **keys;
+	sort_key_t *keys;
 	const char *s, *e;
 	int size = 4;
 	int pos = 0;
 
 	size = 4;
-	keys = xnew(const char *, size);
+	keys = xnew(sort_key_t, size);
 
 	s = value;
 	while (1) {
@@ -232,26 +246,55 @@ static const char **parse_sort_keys(const char *value)
 		s = e;
 
 		for (i = 0; ; i++) {
-			if (valid_sort_keys[i] == NULL) {
+			if (sort_key_map[i].str == NULL) {
 				error_msg("invalid sort key '%s'", buf);
 				free(keys);
 				return NULL;
 			}
 
-			if (strcmp(buf, valid_sort_keys[i]) == 0)
+			if (strcmp(buf, sort_key_map[i].str) == 0)
 				break;
 		}
-
 		if (pos == size - 1) {
 			size *= 2;
-			keys = xrenew(const char *, keys, size);
+			keys = xrenew(sort_key_t, keys, size);
 		}
-		keys[pos++] = valid_sort_keys[i];
+		keys[pos++] = sort_key_map[i].key;
 	}
-	keys[pos] = NULL;
+	keys[pos] = SORT_INVALID;
 	return keys;
 }
 
+static const char *sort_key_to_str(sort_key_t key)
+{
+	int i;
+	for (i = 0; sort_key_map[i].str; i++) {
+		if (sort_key_map[i].key == key)
+			return sort_key_map[i].str;
+	}
+	return NULL;
+}
+
+static void sort_keys_to_str(const sort_key_t *keys, char *buf, size_t bufsize)
+{
+	int i, pos = 0;
+
+	for (i = 0; keys[i] != SORT_INVALID; i++) {
+		const char *key = sort_key_to_str(keys[i]);
+		int len = strlen(key);
+
+		if ((int)bufsize - pos - len - 2 < 0)
+			break;
+
+		memcpy(buf + pos, key, len);
+		pos += len;
+		buf[pos++] = ' ';
+	}
+	if (pos > 0)
+		pos--;
+	buf[pos] = 0;
+}
+
 static void get_lib_sort(unsigned int id, char *buf)
 {
 	strcpy(buf, lib_editable.sort_str);
@@ -259,10 +302,14 @@ static void get_lib_sort(unsigned int id, char *buf)
 
 static void set_lib_sort(unsigned int id, const char *buf)
 {
-	const char **keys = parse_sort_keys(buf);
+	sort_key_t *keys = parse_sort_keys(buf);
 
-	if (keys)
+	if (keys) {
+		editable_lock();
 		editable_set_sort_keys(&lib_editable, keys);
+		editable_unlock();
+		sort_keys_to_str(keys, lib_editable.sort_str, sizeof(lib_editable.sort_str));
+	}
 }
 
 static void get_pl_sort(unsigned int id, char *buf)
@@ -272,10 +319,14 @@ static void get_pl_sort(unsigned int id, char *buf)
 
 static void set_pl_sort(unsigned int id, const char *buf)
 {
-	const char **keys = parse_sort_keys(buf);
+	sort_key_t *keys = parse_sort_keys(buf);
 
-	if (keys)
+	if (keys) {
+		editable_lock();
 		editable_set_sort_keys(&pl_editable, keys);
+		editable_unlock();
+		sort_keys_to_str(keys, pl_editable.sort_str, sizeof(pl_editable.sort_str));
+	}
 }
 
 static void get_output_plugin(unsigned int id, char *buf)
@@ -633,6 +684,21 @@ static void toggle_replaygain_limit(unsigned int id)
 	player_set_rg_limit(replaygain_limit ^ 1);
 }
 
+static void get_resume(unsigned int id, char *buf)
+{
+	strcpy(buf, bool_names[resume_cmus]);
+}
+
+static void set_resume(unsigned int id, const char *buf)
+{
+	parse_bool(buf, &resume_cmus);
+}
+
+static void toggle_resume(unsigned int id)
+{
+	resume_cmus ^= 1;
+}
+
 static void get_show_hidden(unsigned int id, char *buf)
 {
 	strcpy(buf, bool_names[show_hidden]);
@@ -806,7 +872,7 @@ static void set_format(unsigned int id, const char *buf)
 {
 	char **fmtp = id_to_fmt(id);
 
-	if (!format_valid(buf)) {
+	if (!track_format_valid(buf)) {
 		error_msg("invalid format string");
 		return;
 	}
@@ -846,6 +912,7 @@ static const struct {
 	DT(replaygain)
 	DT(replaygain_limit)
 	DN(replaygain_preamp)
+	DT(resume)
 	DT(show_hidden)
 	DT(show_remaining_time)
 	DT(set_term_title)
@@ -893,8 +960,8 @@ static const struct {
 	{ "altformat_title",	"%f" },
 	{ "altformat_trackwin",	" %f%= %d " },
 	{ "format_current",	" %a - %l -%3n. %t%= %y " },
-	{ "format_playlist",	" %-20a %3n. %t%= %y %d " },
-	{ "format_playlist_va",	" %-20A %3n. %t (%a)%= %y %d " },
+	{ "format_playlist",	" %-25%a %3n. %t%= %y %d " },
+	{ "format_playlist_va",	" %-25%A %3n. %t (%a)%= %y %d " },
 	{ "format_title",	"%a - %l - %t (%y)" },
 	{ "format_trackwin",	"%3n. %t%= %y %d " },
 	{ "format_trackwin_va",	"%3n. %t (%a)%= %y %d " },
@@ -1063,3 +1130,133 @@ void options_exit(void)
 
 	fclose(f);
 }
+
+struct resume {
+	enum player_status status;
+	char *filename;
+	long int position;
+	char *lib_filename;
+	int view;
+	char *live_filter;
+};
+
+static int handle_resume_line(void *data, const char *line)
+{
+	struct resume *resume = data;
+	char *cmd, *arg;
+
+	if (!parse_command(line, &cmd, &arg))
+		return 0;
+	if (!arg)
+		goto out;
+
+	if (strcmp(cmd, "status") == 0) {
+		parse_enum(arg, 0, NR_PLAYER_STATUS, player_status_names, (int *) &resume->status);
+	} else if (strcmp(cmd, "file") == 0) {
+		free(resume->filename);
+		resume->filename = xstrdup(unescape(arg));
+	} else if (strcmp(cmd, "position") == 0) {
+		str_to_int(arg, &resume->position);
+	} else if (strcmp(cmd, "lib_file") == 0) {
+		free(resume->lib_filename);
+		resume->lib_filename = xstrdup(unescape(arg));
+	} else if (strcmp(cmd, "view") == 0) {
+		parse_enum(arg, 0, NR_VIEWS, view_names, &resume->view);
+	} else if (strcmp(cmd, "live-filter") == 0) {
+		free(resume->live_filter);
+		resume->live_filter = xstrdup(unescape(arg));
+	}
+
+	free(arg);
+out:
+	free(cmd);
+	return 0;
+}
+
+void resume_load(void)
+{
+	char filename[512];
+	struct track_info *ti, *old;
+	struct resume resume = { .status = PLAYER_STATUS_STOPPED, .view = -1 };
+
+	snprintf(filename, sizeof(filename), "%s/resume", cmus_config_dir);
+	if (file_for_each_line(filename, handle_resume_line, &resume) == -1) {
+		if (errno != ENOENT)
+			error_msg("loading %s: %s", filename, strerror(errno));
+		return;
+	}
+	if (resume.view >= 0 && resume.view != cur_view)
+		set_view(resume.view);
+	if (resume.lib_filename) {
+		cache_lock();
+		ti = old = cache_get_ti(resume.lib_filename);
+		cache_unlock();
+		if (ti) {
+			editable_lock();
+			lib_add_track(ti);
+			track_info_unref(ti);
+			lib_store_cur_track(ti);
+			track_info_unref(ti);
+			ti = lib_set_track(lib_find_track(ti));
+			if (ti) {
+				BUG_ON(ti != old);
+				track_info_unref(ti);
+				tree_sel_current();
+				sorted_sel_current();
+			}
+			editable_unlock();
+		}
+		free(resume.lib_filename);
+	}
+	if (resume.filename) {
+		cache_lock();
+		ti = cache_get_ti(resume.filename);
+		cache_unlock();
+		if (ti) {
+			player_set_file(ti);
+			if (resume.status != PLAYER_STATUS_STOPPED)
+				player_seek(resume.position, 0, resume.status == PLAYER_STATUS_PLAYING);
+		}
+		free(resume.filename);
+	}
+	if (resume.live_filter) {
+		editable_lock();
+		filters_set_live(resume.live_filter);
+		editable_unlock();
+		free(resume.live_filter);
+	}
+}
+
+void resume_exit(void)
+{
+	char filename[512];
+	struct track_info *ti;
+	FILE *f;
+
+	snprintf(filename, sizeof(filename), "%s/resume", cmus_config_dir);
+	f = fopen(filename, "w");
+	if (!f) {
+		warn_errno("creating %s", filename);
+		return;
+	}
+
+	player_info_lock();
+	fprintf(f, "status %s\n", player_status_names[player_info.status]);
+	ti = player_info.ti;
+	if (ti) {
+		fprintf(f, "file %s\n", escape(ti->filename));
+		fprintf(f, "position %d\n", player_info.pos);
+	}
+	player_info_unlock();
+	if (lib_cur_track)
+		ti = tree_track_info(lib_cur_track);
+	else
+		ti = lib_get_cur_stored_track();
+	if (ti)
+		fprintf(f, "lib_file %s\n", escape(ti->filename));
+	fprintf(f, "view %s\n", view_names[cur_view]);
+	if (lib_live_filter)
+		fprintf(f, "live-filter %s\n", escape(lib_live_filter));
+
+	fclose(f);
+}
